section\<open>TLS\<close>
theory TLS12simp
  imports "Automated_Stateful_Protocol_Verification.PSPSP"
begin

declare [[pspsp_timing]]

trac\<open>
Protocol: tls12simp

Types:
hon = {a,b}
dis = {i}
ca  = {s}
agent = hon ++ dis

Sets:
dishonest/0
step1/2
step2/2
step3/2
step4/2
step5/2
data/2

Functions:

Public crypt/2 scrypt/2 tuple/2 exp/2 g/0 h/3 serverK/3 clientK/3 datapack/1
Private inv/1 pk/1 pw/2

Analysis:

tuple(X,Y) -> X,Y
crypt(X,Y) ? inv(X) -> Y
datapack(X) -> X
scrypt(X,Y) ? X -> Y

Transactions:

initialKnowledge(A:hon) 
  send A
  send pk(s).

initialKnowledge2(B:dis)
  send inv(pk(B)).

intruderGen(T:dis)
  new VAL
  insert VAL dishonest
  send VAL.  

# Simplification:
# To be able to omit the first two steps of the original TLS specification we let the intruder
# choose the nonces NA and NB
client_key_ex(A:hon,B:hon,NA:value,NB:value)
  receive NA
  receive NB
  new PMS
  insert PMS step3(A,B)
  send crypt(pk(B),PMS)
  send scrypt(clientK(PMS,NA,NB),h(PMS,NA,NB)).
  
server_key_ex(A:hon,B:hon,NA:value,NB:value,PMS:value)
  receive crypt(pk(B),PMS)
  receive scrypt(clientK(PMS,NA,NB),h(PMS,NA,NB))
  insert PMS step4(A,B)
  send scrypt(serverK(PMS,NA,NB),h(PMS,NA,NB)).

clientFinishedData(A:hon,B:hon,NA:value,NB:value,PMS:value)
  receive scrypt(serverK(PMS,NA,NB),h(PMS,NA,NB))
  PMS in step3(A,B)
  new Data
  insert Data data(A,B)
  send scrypt(clientK(PMS,NA,NB), tuple(pw(A,B),datapack(Data))).

serverData(A:hon,B:hon,NA:value,NB:value,PMS:value,Data:value)
  receive scrypt(clientK(PMS,NA,NB), tuple(pw(A,B),datapack(Data)))
  PMS in step4(A,B)
  new DataB
  insert DataB data(B,A)
  send scrypt(serverK(PMS,NA,NB), datapack(DataB)).

### Attack

attackRule(A:hon,B:hon,Data:value)
  receive Data
  Data in data(A,B)
  attack.
\<close>


subsection \<open>Proof of security\<close>
protocol_model_setup spm: tls12simp
compute_fixpoint tls12simp_protocol tls12simp_fixpoint
value "let (FP,_,TI) = tls12simp_fixpoint in (size FP, size TI)"
protocol_security_proof [unsafe] ssp: tls12simp


subsection \<open>The generated theorems and definitions\<close>
thm ssp.protocol_secure

thm tls12simp_enum_consts.nchotomy
thm tls12simp_sets.nchotomy
thm tls12simp_fun.nchotomy
thm tls12simp_atom.nchotomy
thm tls12simp_arity.simps
thm tls12simp_public.simps
thm tls12simp_\<Gamma>.simps
thm tls12simp_Ana.simps

thm tls12simp_protocol_def

end
